=== modified file 'plugins.d/mandos-client.c'
--- plugins.d/mandos-client.c	2011-11-11 09:07:09 +0000
+++ plugins.d/mandos-client.c	2011-10-05 16:00:56 +0000
@@ -1085,84 +1085,107 @@
   errno = old_errno;
 }
 
-bool get_flags(const char *ifname, struct ifreq *ifr){
-  int ret;
-  
-  int s = socket(PF_INET6, SOCK_DGRAM, IPPROTO_IP);
-  if(s < 0){
-    perror_plus("socket");
-    return false;
-  }
-  strcpy(ifr->ifr_name, ifname);
-  ret = ioctl(s, SIOCGIFFLAGS, ifr);
-  if(ret == -1){
+/* 
+ * This function determines if a directory entry in /sys/class/net
+ * corresponds to an acceptable network device.
+ * (This function is passed to scandir(3) as a filter function.)
+ */
+int good_interface(const struct dirent *if_entry){
+  ssize_t ssret;
+  char *flagname = NULL;
+  if(if_entry->d_name[0] == '.'){
+    return 0;
+  }
+  int ret = asprintf(&flagname, "%s/%s/flags", sys_class_net,
+		     if_entry->d_name);
+  if(ret < 0){
+    perror_plus("asprintf");
+    return 0;
+  }
+  int flags_fd = (int)TEMP_FAILURE_RETRY(open(flagname, O_RDONLY));
+  if(flags_fd == -1){
+    perror_plus("open");
+    free(flagname);
+    return 0;
+  }
+  free(flagname);
+  typedef short ifreq_flags;	/* ifreq.ifr_flags in netdevice(7) */
+  /* read line from flags_fd */
+  ssize_t to_read = 2+(sizeof(ifreq_flags)*2)+1; /* "0x1003\n" */
+  char *flagstring = malloc((size_t)to_read+1); /* +1 for final \0 */
+  flagstring[(size_t)to_read] = '\0';
+  if(flagstring == NULL){
+    perror_plus("malloc");
+    close(flags_fd);
+    return 0;
+  }
+  while(to_read > 0){
+    ssret = (ssize_t)TEMP_FAILURE_RETRY(read(flags_fd, flagstring,
+					     (size_t)to_read));
+    if(ssret == -1){
+      perror_plus("read");
+      free(flagstring);
+      close(flags_fd);
+      return 0;
+    }
+    to_read -= ssret;
+    if(ssret == 0){
+      break;
+    }
+  }
+  close(flags_fd);
+  intmax_t tmpmax;
+  char *tmp;
+  errno = 0;
+  tmpmax = strtoimax(flagstring, &tmp, 0);
+  if(errno != 0 or tmp == flagstring or (*tmp != '\0'
+					 and not (isspace(*tmp)))
+     or tmpmax != (ifreq_flags)tmpmax){
     if(debug){
-      perror_plus("ioctl SIOCGIFFLAGS");
+      fprintf(stderr, "Invalid flags \"%s\" for interface \"%s\"\n",
+	      flagstring, if_entry->d_name);
     }
-    return false;
+    free(flagstring);
+    return 0;
   }
-  return true;
-}
-
-bool good_flags(const char *ifname, const struct ifreq *ifr){
-  
+  free(flagstring);
+  ifreq_flags flags = (ifreq_flags)tmpmax;
   /* Reject the loopback device */
-  if(ifr->ifr_flags & IFF_LOOPBACK){
+  if(flags & IFF_LOOPBACK){
     if(debug){
       fprintf(stderr, "Rejecting loopback interface \"%s\"\n",
-	      ifname);
+	      if_entry->d_name);
     }
-    return false;
+    return 0;
   }
   /* Accept point-to-point devices only if connect_to is specified */
-  if(connect_to != NULL and (ifr->ifr_flags & IFF_POINTOPOINT)){
+  if(connect_to != NULL and (flags & IFF_POINTOPOINT)){
     if(debug){
       fprintf(stderr, "Accepting point-to-point interface \"%s\"\n",
-	      ifname);
+	      if_entry->d_name);
     }
-    return true;
+    return 1;
   }
   /* Otherwise, reject non-broadcast-capable devices */
-  if(not (ifr->ifr_flags & IFF_BROADCAST)){
+  if(not (flags & IFF_BROADCAST)){
     if(debug){
       fprintf(stderr, "Rejecting non-broadcast interface \"%s\"\n",
-	      ifname);
+	      if_entry->d_name);
     }
-    return false;
+    return 0;
   }
   /* Reject non-ARP interfaces (including dummy interfaces) */
-  if(ifr->ifr_flags & IFF_NOARP){
+  if(flags & IFF_NOARP){
     if(debug){
-      fprintf(stderr, "Rejecting non-ARP interface \"%s\"\n", ifname);
+      fprintf(stderr, "Rejecting non-ARP interface \"%s\"\n",
+	      if_entry->d_name);
     }
-    return false;
+    return 0;
   }
-  
   /* Accept this device */
   if(debug){
-    fprintf(stderr, "Interface \"%s\" is good\n", ifname);
-  }
-  return true;
-}
-
-/* 
- * This function determines if a directory entry in /sys/class/net
- * corresponds to an acceptable network device.
- * (This function is passed to scandir(3) as a filter function.)
- */
-int good_interface(const struct dirent *if_entry){
-  int ret;
-  if(if_entry->d_name[0] == '.'){
-    return 0;
-  }
-  struct ifreq ifr;
-
-  if(not get_flags(if_entry->d_name, &ifr)){
-    return 0;
-  }
-  
-  if(not good_flags(if_entry->d_name, &ifr)){
-    return 0;
+    fprintf(stderr, "Interface \"%s\" is acceptable\n",
+	    if_entry->d_name);
   }
   return 1;
 }

